/*******************************************************************************
 * Copyright (c) 2014-2015 Marian Schedenig
 * 
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Marian Schedenig - initial API and implementation
 *******************************************************************************/

package name.schedenig.eclipse.clippets.c.utils;

import name.schedenig.eclipse.clippets.c.Activator;

import org.eclipse.cdt.core.model.CModelException;
import org.eclipse.cdt.core.model.ICElement;
import org.eclipse.cdt.core.model.IFunction;
import org.eclipse.cdt.core.model.IMethod;
import org.eclipse.cdt.core.model.IStructure;
import org.eclipse.cdt.core.model.ITranslationUnit;
import org.eclipse.cdt.ui.CDTUITools;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.jface.text.ITextSelection;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.texteditor.ITextEditor;

public class CUtils
{
	public static IStructure getCStructure(ITextEditor editor, int type)
	{
		return (IStructure) getCElement(editor, type);
	}
	
	public static IMethod getCMethod(ITextEditor editor)
	{
		return (IMethod) getCElement(editor, ICElement.C_METHOD);
	}
	
	public static IFunction getCFunction(ITextEditor editor)
	{
		return (IFunction) getCElement(editor, ICElement.C_FUNCTION);
	}
	
	public static ICElement getCElement(ITextEditor editor, int type)
	{
		if(editor == null)
		{
			return null;
		}
		
		ITextSelection selection = (ITextSelection) editor.getSelectionProvider().getSelection();
		IEditorInput editorInput = editor.getEditorInput();

		ICElement elem = CDTUITools.getEditorInputCElement(editorInput);
		
		if(!(elem instanceof ITranslationUnit))
		{
			return null;
		}
		
		ITranslationUnit unit = (ITranslationUnit) elem;
		
		try
		{
			ICElement selected = unit.getElementAtOffset(selection.getOffset());
			
			if(selected == null)
			{
				return null;
			}
			
			return selected.getAncestor(type);
		}
		catch(CModelException ex)
		{
			Activator.getDefault().log(IStatus.ERROR, ex);
			return null;
		}
		
		//return null;//elem.getElementName(); // TODO
	}
}
